package edu.cmu.paid.learning.dataminer.developer.NaiveBayes;

import java.util.Hashtable;


/** 
 * NormalDistribution is a class that implements algorithm to calculate the
 * probability given a normal distribution w/ mean and standard deviation.
 *
 * @author Rudy Setiawan, PAID Learning Team, Carnegie Mellon University.
 * @version %I%, %G%
 */
public class NormalDistribution implements java.io.Serializable
{
  private double mean;
  private double stdev;
  private static Hashtable ndLookup = generateLookupTable();

  /**
   * The constructor for NormalDistribution takes the average and standard
   * deviation values.
   *
   * @param avg the mean
   * @param std the standard deviation
   */
  public NormalDistribution(double avg, double std)
  {
    mean = avg;
    stdev = std;
    
    // generateLookupTable();
  }

  
  /**
   * Calculates probability that a number falls below or equal to given value.
   *
   * @param value the value to be evaluated
   * @return the probability
   */
  public double evaluate(double value)
  {
    double result = 0;

    // normalized for using standardized normal distribution
    double z = (Math.abs(value - mean))/stdev;

    // round off number to 2 significant digits, so can be used as hash key
    z*= 100;
    int zInt = (int) z;
    z = (double) zInt/100;

    double lookup;

    if(ndLookup.containsKey(new Double(z)))
      lookup = ((Double)ndLookup.get(new Double(z))).doubleValue();
    else
      return(-1);

    // System.out.println("z is " + z);
    // System.out.println("lookup is " + lookup);

    if(value > mean)
      result = 0.5 + lookup;
    else
      result = 0.5 - lookup;

    return result;
  }


  /**
   * Generates a lookup table to be used in evaluation later on. The reason i
   * to speed up calculation, not having to do it for each evaluation (costly,
   * involving integration)
   *
   * @return the lookup table
   */
  static Hashtable generateLookupTable()
  {
    Hashtable toBeReturned = new Hashtable();

    double total = 0;
    
    double i = 0;

    while(total <= .5)
      {
	// integration with interval size 0.0001 (smaller for better accuracy)
	double z = 0.0001 * i;
	double value = normalDensity(z) * 0.0001;

	z*= 100;

	int zInt = (int) z;

	z = (double) zInt/100;

	total+= value;

	if((i%100) == 0)
	  {
	    toBeReturned.put(new Double(z), new Double(total));
	  }
	i++;
      }

    return toBeReturned;
  }

  static double normalDensity(double value)
  {
    double result = 0;

    result = (Math.exp(-((Math.pow(value, 2))/2)))/(Math.sqrt(2 * Math.PI));

    return result;
  }


  public String toString()
  {
    String toBeReturned = "";

    toBeReturned += "mean: " + mean;
    toBeReturned += ", stdev: " + stdev;

    return toBeReturned;
  }


  // test program
  public static void main(String[] args)
  {
    double mean = Double.valueOf(args[0]).doubleValue();
    double std = Double.valueOf(args[1]).doubleValue();
    NormalDistribution prob1 = new NormalDistribution(mean, std);

    double result = 0;

    result = prob1.evaluate(Double.valueOf(args[2]).doubleValue());

    System.out.println("Result is " + result);
  }  
}
